@@ -2,7 +2,7 @@
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 
 // This program is distributed in the hope that it will be useful,
@@ -18,13 +18,14 @@
 // Based on sound_handler_sdl.cpp by Thatcher Ulrich http://tulrich.com 2003
 // which has been donated to the Public Domain.
 
-// $Id: sound_handler_sdl.cpp,v 1.68.4.1 2007/06/08 11:40:41 tgc Exp $
+// $Id: sound_handler_sdl.cpp,v 1.79.2.1 2007/08/15 15:02:41 tgc Exp $
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include "sound_handler_sdl.h"
+#include "utility.h" // for convert_raw_data
 
 #include "log.h"
 #include <cmath>
@@ -33,7 +34,7 @@
 
 using namespace boost;
 
-static void sdl_audio_callback(void *udata, Uint8 *stream, int len); // SDL C audio handler
+namespace gnash {
 
 SDL_sound_handler::SDL_sound_handler()
 	: soundOpened(false),
@@ -44,7 +45,7 @@ SDL_sound_handler::SDL_sound_handler()
 	audioSpec.freq = 44100;
 	audioSpec.format = AUDIO_S16SYS; // AUDIO_S8 AUDIO_U8;
 	audioSpec.channels = 2;
-	audioSpec.callback = sdl_audio_callback;
+	audioSpec.callback = SDL_sound_handler::sdl_audio_callback;
 	audioSpec.userdata = this;
 	audioSpec.samples = 2048;		//512 - not enough for  videostream
 }
@@ -73,7 +74,7 @@ int	SDL_sound_handler::create_sound(
 
 	sound_data *sounddata = new sound_data;
 	if (!sounddata) {
-		gnash::log_error(_("could not allocate memory for sound data"));
+		log_error(_("could not allocate memory for sound data"));
 		return -1;
 	}
 
@@ -97,7 +98,7 @@ int	SDL_sound_handler::create_sound(
 			convert_raw_data(&adjusted_data, &adjusted_size, data, sample_count, 2, sample_rate, stereo,
 					 audioSpec.freq, (audioSpec.channels == 2 ? true : false));
 			if (!adjusted_data) {
-				gnash::log_error(_("Some kind of error occurred with sound data"));
+				log_error(_("Some kind of error occurred with sound data"));
 				return -1;
 			}
 			sounddata->data_size = adjusted_size;
@@ -108,13 +109,13 @@ int	SDL_sound_handler::create_sound(
 	case FORMAT_MP3:
 #ifndef USE_FFMPEG
 #ifndef USE_MAD
-		gnash::log_error(_("gnash has not been compiled to handle mp3 audio"));
+		log_error(_("gnash has not been compiled to handle mp3 audio"));
 		return -1;
 #endif
 #endif
 		sounddata->data = new Uint8[data_bytes];
 		if (!sounddata->data) {
-			gnash::log_error(_("could not allocate space for data in sound handler"));
+			log_error(_("could not allocate space for data in sound handler"));
 			return -1;
 		}
 		memcpy(sounddata->data, data, data_bytes);
@@ -126,17 +127,17 @@ int	SDL_sound_handler::create_sound(
 	case FORMAT_ADPCM:
 	case FORMAT_UNCOMPRESSED:
 		// These should have been converted to FORMAT_NATIVE16
-		gnash::log_error(_("Sound data format not properly converted"));
+		log_error(_("Sound data format not properly converted"));
 		return -1;
 		break;
 
 	case FORMAT_NELLYMOSER:
-		gnash::log_unimpl("Nellymoser sound format requested, gnash does not handle it.");
+		log_unimpl("Nellymoser sound format requested, gnash does not handle it.");
 		return -1;
 
 	default:
 		// Unhandled format.
-		gnash::log_error(_("unknown sound format %d requested; gnash does not handle it"), (int)format);
+		log_error(_("unknown sound format %d requested; gnash does not handle it"), (int)format);
 		return -1; // Unhandled format, set to NULL.
 	}
 
@@ -172,7 +173,7 @@ long	SDL_sound_handler::fill_stream_data(void* data, int data_bytes, int sample_
 				 sounddata->sample_rate, sounddata->stereo,
 				 audioSpec.freq, (audioSpec.channels == 2));
 		if (!adjusted_data || adjusted_size < 1) {
-			gnash::log_error(_("Some kind of error with resampling sound data"));
+			log_error(_("Some kind of error with resampling sound data"));
 			return -1;
 		}
 
@@ -220,7 +221,7 @@ long	SDL_sound_handler::fill_stream_data(void* data, int data_bytes, int sample_
 	    break;
 
 	default:
-		gnash::log_error(_("Behavior for this audio codec %d is unknown.  Please send this SWF to the developers"), (int)(sounddata->format));
+		log_error(_("Behavior for this audio codec %d is unknown.  Please send this SWF to the developers"), (int)(sounddata->format));
 	}
 
 	return start_size;
@@ -242,8 +243,7 @@ void	SDL_sound_handler::play_sound(int sound_handle, int loop_count, int offset,
 	sound_data* sounddata = m_sound_data[sound_handle];
 
 	// If this is called from a streamsoundblocktag, we only start if this
-	// sound isn't already playing. If a active_sound-struct is existing we
-	// assume it is also playing.
+	// sound isn't already playing.
 	if (start_position > 0 && sounddata->m_active_sounds.size() > 0) {
 		return;
 	}
@@ -251,7 +251,7 @@ void	SDL_sound_handler::play_sound(int sound_handle, int loop_count, int offset,
 	// Make sure sound actually got some data
 	if (sounddata->data_size < 1) {
 		IF_VERBOSE_MALFORMED_SWF(
-			gnash::log_swferror(_("Trying to play sound with size 0"));
+			log_swferror(_("Trying to play sound with size 0"));
 		);
 		return;
 	}
@@ -289,7 +289,7 @@ void	SDL_sound_handler::play_sound(int sound_handle, int loop_count, int offset,
 		sound->parser = av_parser_init(CODEC_ID_MP3);
 
 		if (!sound->codec) {
-			gnash::log_error(_("Your FFMPEG can't decode MP3?!"));
+			log_error(_("Your FFMPEG can't decode MP3?!"));
 			return;
 		}
 
@@ -318,7 +318,7 @@ void	SDL_sound_handler::play_sound(int sound_handle, int loop_count, int offset,
 
 	if (!soundOpened) {
 		if (SDL_OpenAudio(&audioSpec, NULL) < 0 ) {
-			gnash::log_error(_("Unable to start SDL sound: %s"), SDL_GetError());
+			log_error(_("Unable to start SDL sound: %s"), SDL_GetError());
 			return;
 		}
 		soundOpened = true;
@@ -377,14 +377,15 @@ void	SDL_sound_handler::stop_sound(int sound_handle)
 }
 
 
-void	SDL_sound_handler::delete_sound(int sound_handle)
 // this gets called when it's done with a sample.
+void	SDL_sound_handler::delete_sound(int sound_handle)
 {
 	mutex::scoped_lock lock(_mutex);
 
 	if (sound_handle >= 0 && static_cast<unsigned int>(sound_handle) < m_sound_data.size())
 	{
-		delete[] m_sound_data[sound_handle]->data;
+		delete m_sound_data[sound_handle];
+		m_sound_data[sound_handle] = NULL;
 	}
 
 }
@@ -512,7 +513,7 @@ void	SDL_sound_handler::attach_aux_streamer(aux_streamer_ptr ptr, void* owner)
 
 	if (!soundOpened) {
 		if (SDL_OpenAudio(&audioSpec, NULL) < 0 ) {
-			gnash::log_error(_("Unable to start aux SDL sound: %s"), SDL_GetError());
+			log_error(_("Unable to start aux SDL sound: %s"), SDL_GetError());
 			return;
 		}
 		soundOpened = true;
@@ -533,8 +534,58 @@ void	SDL_sound_handler::detach_aux_streamer(void* owner)
 
 }
 
+unsigned int SDL_sound_handler::get_duration(int sound_handle)
+{
+	mutex::scoped_lock lock(_mutex);
 
-gnash::sound_handler*	gnash::create_sound_handler_sdl()
+	// Check if the sound exists.
+	if (sound_handle < 0 || (unsigned int) sound_handle >= m_sound_data.size())
+	{
+		// Invalid handle.
+		return 0;
+	}
+
+	sound_data* sounddata = m_sound_data[sound_handle];
+
+	// Return the sound duration in milliseconds
+	if (sounddata->sample_count > 0 && sounddata->sample_rate > 0) {
+		unsigned int ret = sounddata->sample_count / sounddata->sample_rate * 1000;
+		ret += ((sounddata->sample_count % sounddata->sample_rate) * 1000) / sounddata->sample_rate;
+		if (sounddata->stereo) ret = ret / 2;
+		return ret;
+	} else {
+		return 0;
+	}
+}
+
+unsigned int SDL_sound_handler::get_position(int sound_handle)
+{
+	mutex::scoped_lock lock(_mutex);
+
+	// Check if the sound exists.
+	if (sound_handle < 0 || (unsigned int) sound_handle >= m_sound_data.size())
+	{
+		// Invalid handle.
+		return 0;
+	}
+
+	sound_data* sounddata = m_sound_data[sound_handle];
+
+	// If there is no active sounds, return 0
+	if (sounddata->m_active_sounds.size() == 0) return 0;
+
+	// We use the first active sound of this.
+	active_sound* asound = sounddata->m_active_sounds[0];
+
+	// Return the playhead position in milliseconds
+	unsigned int ret = asound->samples_played / audioSpec.freq * 1000;
+	ret += ((asound->samples_played % audioSpec.freq) * 1000) / audioSpec.freq;
+	if (audioSpec.channels > 1) ret = ret / audioSpec.channels;
+	return ret;
+}
+
+sound_handler*
+create_sound_handler_sdl()
 // Factory.
 {
 	return new SDL_sound_handler;
@@ -646,50 +697,24 @@ do_mixing(Uint8* stream, active_sound* sound, Uint8* data, unsigned int mix_leng
 
 	// Update sound info
 	sound->raw_position += mix_length;
-	sound->samples_played += mix_length;
+
+	// Sample size is always 2
+	sound->samples_played += mix_length / 2;
 }
 
 
-/// Callback invoked by the SDL audio thread.
-//
-/// Refills the output stream/buffer with data.
-///
-/// We run trough all the attached auxiliary streamers fetching decoded
-/// audio blocks and mixing them into the given output stream.
-///
-/// <UnverifiedComment>
-///   If sound is compresssed (mp3) a mp3-frame is decoded into a buffer,
-///   and resampled if needed. When the buffer has been sampled, another
-///   frame is decoded until all frames has been decoded.
-///   If a sound is looping it will be decoded from the beginning again.
-/// </UnverifiedComment>
-///
-/// TODO: make a static method of the SDL_sound_handler class
-///
-/// @param udata
-///	User data pointer (SDL_sound_handler instance in our case).
-///	We'll lock the SDL_sound_handler::_mutex during operations.
-///
-/// @param stream
-/// 	The output stream/buffer to fill
-///
-/// @param buffer_length_in
-///	Length of the buffer.
-///	If zero or negative we log an error and return
-///	(negative is probably an SDL bug, zero dunno yet).
-///
-static void
-sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
+// Callback invoked by the SDL audio thread.
+void SDL_sound_handler::sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 {
 	if ( buffer_length_in < 0 )
 	{
-		gnash::log_error(_("Negative buffer length in sdl_audio_callback (%d)"), buffer_length_in);
+		log_error(_("Negative buffer length in sdl_audio_callback (%d)"), buffer_length_in);
 		return;
 	}
 
 	if ( buffer_length_in == 0 )
 	{
-		gnash::log_error(_("Zero buffer length in sdl_audio_callback"));
+		log_error(_("Zero buffer length in sdl_audio_callback"));
 		return;
 	}
 
@@ -714,11 +739,11 @@ sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 	// call NetStream or Sound audio callbacks
 	if (handler->m_aux_streamer.size() > 0)
 	{
-		Uint8* buf = new Uint8[buffer_length];
+		uint8_t* buf = new uint8_t[buffer_length];
 
 		// Loop through the attached sounds
-		hash_wrapper< void*, gnash::sound_handler::aux_streamer_ptr >::iterator it = handler->m_aux_streamer.begin();
-		hash_wrapper< void*, gnash::sound_handler::aux_streamer_ptr >::iterator end = handler->m_aux_streamer.end();
+		hash_wrapper< void*, sound_handler::aux_streamer_ptr >::iterator it = handler->m_aux_streamer.begin();
+		hash_wrapper< void*, sound_handler::aux_streamer_ptr >::iterator end = handler->m_aux_streamer.end();
 		while (it != end) {
 			memset(buf, 0, buffer_length);
 
@@ -739,10 +764,7 @@ sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 		delete [] buf;
 	}
 
-#ifdef WIN32	// hack
-	return;
-#endif
-
+	// Run through all the sounds.
 	for(uint32_t i=0; i < handler->m_sound_data.size(); i++) {
 		sound_data* sounddata = handler->m_sound_data[i];
 		for(uint32_t j = 0; j < sounddata->m_active_sounds.size(); j++) {
@@ -772,6 +794,7 @@ sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 					if (index < buffer_length) {
 						sound->loop_count--;
 						sound->raw_position = 0;
+						sound->samples_played = 0;
 						unsigned int mix_length = ((buffer_length - index) > sound->raw_data_size ? sound->raw_data_size : (buffer_length - index));
 						do_mixing(stream+index, sound, sound->get_raw_data_ptr(sound->raw_position), 
 							mix_length, sounddata->volume);
@@ -790,6 +813,7 @@ sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 					if (sound->data_size == sound->position && sound->loop_count != 0) {
 						sound->loop_count--;
 						sound->position = 0;
+						sound->samples_played = 0;
 					}
 
 					// Test if we will get problems... Should not happen...
@@ -806,7 +830,7 @@ sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 
 					long bytes_decoded = 0;
 
-					while (outsize == 0) {
+					while (outsize == 0 && sound->position < sound->data_size) {
 						uint8_t* frame;
 						int framesize;
 
@@ -823,7 +847,7 @@ sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 #endif
 
 						if (bytes_decoded < 0 || tmp < 0 || outsize < 0) {
-							gnash::log_error(_("Error while decoding MP3-stream.  Upgrading ffmpeg/libavcodec might fix this issue."));
+							log_error(_("Error while decoding MP3-stream.  Upgrading ffmpeg/libavcodec might fix this issue."));
 							// Setting data position to data size will get the sound removed
 							// from the active sound list later on.
 							sound->position = sound->data_size;
@@ -853,7 +877,7 @@ sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 						// Error handling is done by relooping (max. 8 times) and just hooping that it will work...
 						if (loops > 8) break;
 						if (ret == -1 && sound->stream.error != MAD_ERROR_BUFLEN && MAD_RECOVERABLE(sound->stream.error)) {
-							gnash::log_error(_("Recoverable error while decoding MP3-stream, MAD error: %s"), mad_stream_errorstr (&sound->stream));
+							log_error(_("Recoverable error while decoding MP3-stream, MAD error: %s"), mad_stream_errorstr (&sound->stream));
 							continue;
 						}
 						
@@ -861,7 +885,7 @@ sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 					}
 
 					if (ret == -1 && sound->stream.error != MAD_ERROR_BUFLEN) {
-						gnash::log_error(_("Unrecoverable error while decoding MP3-stream, MAD error: %s"), mad_stream_errorstr (&sound->stream));
+						log_error(_("Unrecoverable error while decoding MP3-stream, MAD error: %s"), mad_stream_errorstr (&sound->stream));
 						sound->position = sound->data_size;
 						continue;
 					} else if (ret == -1 && sound->stream.error == MAD_ERROR_BUFLEN) {
@@ -913,13 +937,13 @@ sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 						int sample_count = outsize / ((sounddata->stereo == true) ? 4 : 2);
 
 						// Convert to needed samplerate
-						handler->convert_raw_data(&adjusted_data, &adjusted_size, tmp_raw_buffer, sample_count, 0, 
+						convert_raw_data(&adjusted_data, &adjusted_size, tmp_raw_buffer, sample_count, 0, 
 								sounddata->sample_rate, sounddata->stereo,
 								handler->audioSpec.freq, (handler->audioSpec.channels == 2 ? true : false));
 
 						// Hopefully this wont happen
 						if (!adjusted_data) {
-							gnash::log_error(_("Error in sound sample conversion"));
+							log_error(_("Error in sound sample conversion"));
 							continue;
 						}
 
@@ -959,7 +983,7 @@ sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 				} else { 
 					mix_length = decoded_size;
 				}
-
+				if (sound->raw_data_size < 2) continue; // something went terrible wrong
 				do_mixing(stream+index, sound, sound->get_raw_data_ptr(0), mix_length, sounddata->volume);
 
 			// When the current sound has enough decoded data to fill 
@@ -1005,11 +1029,13 @@ sdl_audio_callback (void *udata, Uint8 *stream, int buffer_length_in)
 				handler->soundsPlaying--;
 			}
 
-		}
-	}
+		} // active sounds loop
+	} // existing sounds loop
 
 }
 
+} // namespace gnash
+
 // Local Variables:
 // mode: C++
 // End:
