@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -18,12 +18,13 @@
 // 
 //
 
-/* $Id: aqua_glue.h,v 1.4 2007/05/28 15:40:59 ann Exp $ */
+/* $Id: aqua_glue.h,v 1.8 2007/07/24 00:04:39 nihilus Exp $ */
 
 #ifndef AQUA_GLUE_H
 #define AQUA_GLUE_H
 
 #include "gnash.h"
+#include "snappingrange.h"
 
 namespace gnash
 {
@@ -33,9 +34,8 @@ class AquaGlue
   public:
     virtual ~AquaGlue() { }
     virtual bool init(int argc, char **argv[]) = 0;
-
     virtual bool prepDrawingArea(int width, int height) = 0;
-    virtual render_handler* createRenderHandler(int depth) = 0;
+    virtual render_handler* createRenderHandler() = 0;
     virtual void render() = 0;
   protected:
     int _bpp;
