@@ -11,6 +11,7 @@
 // converted from K&R C to C-like C++, changed the interfaces a bit,
 // etc.
 
+/* $Id: image_filters.cpp,v 1.15 2007/06/16 12:47:54 strk Exp $ */
 
 #include "image.h"
 #include "utility.h"
@@ -18,9 +19,11 @@
 #include "tu_math.h"
 #include <cstdio>
 #include <cstring>
+#include <cmath>
 #include "log.h"
 
 using namespace gnash;
+using namespace std;
 
 namespace {
 // anonymous namespace to hold local stuff.
@@ -39,7 +42,7 @@ inline void	my_cfree(void* mem)
 }
 
 
-void	get_row(Uint8* row, image::rgb* image, int x0, int xsize, int y)
+void	get_row(uint8_t* row, image::rgb* image, int x0, int xsize, int y)
 // Copy RGB data from the specified row into the given buffer.
 {
     y = iclamp(y, 0, image->m_height - 1);
@@ -47,11 +50,11 @@ void	get_row(Uint8* row, image::rgb* image, int x0, int xsize, int y)
     if (x1 >= image->m_width) {
 	// clip, then extend.
 	int	extra_pixels = x1 - image->m_width + 1;
-	Uint8*	p = ((Uint8*) image->m_data) + (y * image->m_pitch);
+	uint8_t*	p = ((uint8_t*) image->m_data) + (y * image->m_pitch);
 	memcpy(row, p + x0 * 3, (3 * (image->m_width - x0)));
 	// repeat last pixel
 	p = p + (image->m_width - 1) * 3;
-	Uint8*	q = row + (image->m_width - x0) * 3;
+	uint8_t*	q = row + (image->m_width - x0) * 3;
 	while (extra_pixels > 0) {
 	    *(q + 0) = *(p + 0);
 	    *(q + 1) = *(p + 1);
@@ -62,12 +65,12 @@ void	get_row(Uint8* row, image::rgb* image, int x0, int xsize, int y)
     }
     else
 	{
-	    memcpy(row, ((Uint8*) image->m_data) + (y * image->m_pitch) + x0 * 3, (3 * xsize));
+	    memcpy(row, ((uint8_t*) image->m_data) + (y * image->m_pitch) + x0 * 3, (3 * xsize));
 	}
 }
 
 
-void	get_row(Uint8* row, image::rgba* image, int x0, int xsize, int y)
+void	get_row(uint8_t* row, image::rgba* image, int x0, int xsize, int y)
 // Copy RGBA data from the specified row into the given buffer.
 {
     y = iclamp(y, 0, image->m_height - 1);
@@ -75,11 +78,11 @@ void	get_row(Uint8* row, image::rgba* image, int x0, int xsize, int y)
     if (x1 >= image->m_width) {
 	// clip, then extend.
 	int	extra_pixels = x1 - image->m_width + 1;
-	Uint8*	p = ((Uint8*) image->m_data) + (y * image->m_pitch);
+	uint8_t*	p = ((uint8_t*) image->m_data) + (y * image->m_pitch);
 	memcpy(row, p + x0 * 4, (4 * (image->m_width - x0)));
 	// repeat last pixel
 	p = p + (image->m_width - 1) * 4;
-	Uint8*	q = row + (image->m_width - x0) * 4;
+	uint8_t*	q = row + (image->m_width - x0) * 4;
 	while (extra_pixels > 0) {
 	    *(q + 0) = *(p + 0);
 	    *(q + 1) = *(p + 1);
@@ -91,24 +94,23 @@ void	get_row(Uint8* row, image::rgba* image, int x0, int xsize, int y)
     }
     else
 	{
-	    memcpy(row, ((Uint8*) image->m_data) + (y * image->m_pitch) + x0 * 4, (4 * xsize));
+	    memcpy(row, ((uint8_t*) image->m_data) + (y * image->m_pitch) + x0 * 4, (4 * xsize));
 	}
 }
 
 
-void	get_column(Uint8* column, image::rgb* image, int x)
+void	get_column(uint8_t* column, image::rgb* image, int x)
 // Copy RGB data from the specified column into the given buffer.
 {
-    int	i, d;
-    Uint8*	p;
 
     if ((x < 0) || (x >= image->m_width)) {
 	assert(0);
 	x = iclamp(x, 0, image->m_width - 1);
     }
 
-    d = image->m_pitch;
-    for (i = image->m_height, p = ((Uint8*) image->m_data) + x * 3; i-- > 0; p += d) {
+    int d = image->m_pitch;
+    uint8_t* p = ((uint8_t*) image->m_data) + x * 3;
+    for (int i = image->m_height; i-- > 0; p += d) {
 	*column++ = *p;
 	*column++ = *(p + 1);
 	*column++ = *(p + 2);
@@ -116,19 +118,17 @@ void	get_column(Uint8* column, image::rgb* image, int x)
 }
 
 
-void	get_column(Uint8* column, image::rgba* image, int x)
+void	get_column(uint8_t* column, image::rgba* image, int x)
 // Copy RGBA data from the specified column into the given buffer.
 {
-    int	i, d;
-    Uint8*	p;
-
     if ((x < 0) || (x >= image->m_width)) {
 	assert(0);
 	x = iclamp(x, 0, image->m_width - 1);
     }
 
-    d = image->m_pitch;
-    for (i = image->m_height, p = ((Uint8*) image->m_data) + x * 4; i-- > 0; p += d) {
+    int d = image->m_pitch;
+    uint8_t* p = ((uint8_t*) image->m_data) + x * 4;
+    for (int i = image->m_height; i-- > 0; p += d) {
 	*column++ = *p;
 	*column++ = *(p + 1);
 	*column++ = *(p + 2);
@@ -142,8 +142,8 @@ void	put_pixel(image::rgb* image, int x, int y, float r, float g, float b)
 // at (x, y).
 {
     static image::rgb*	im = NULL;
-    static int	yy = -1;
-    static Uint8*	p = NULL;
+    static int		yy = -1;
+    static uint8_t*	p = NULL;
 
     if ((x < 0) || (x >= image->m_width) || (y < 0) || (y >= image->m_height)) {
 	assert(0);
@@ -152,7 +152,7 @@ void	put_pixel(image::rgb* image, int x, int y, float r, float g, float b)
     if ((im != image) || (yy != y)) {
 	im = image;
 	yy = y;
-	p = ((Uint8*) image->m_data) + (y * image->m_pitch);
+	p = ((uint8_t*) image->m_data) + (y * image->m_pitch);
     }
     p[x * 3 + 0] = iclamp(frnd(r), 0, 255);
     p[x * 3 + 1] = iclamp(frnd(g), 0, 255);
@@ -165,8 +165,8 @@ void	put_pixel(image::rgba* image, int x, int y, float r, float g, float b, floa
 // at (x, y).
 {
     static image::rgba*	im = NULL;
-    static int	yy = -1;
-    static Uint8*	p = NULL;
+    static int		yy = -1;
+    static uint8_t*	p = NULL;
 
     if ((x < 0) || (x >= image->m_width) || (y < 0) || (y >= image->m_height)) {
 	assert(0);
@@ -175,7 +175,7 @@ void	put_pixel(image::rgba* image, int x, int y, float r, float g, float b, floa
     if ((im != image) || (yy != y)) {
 	im = image;
 	yy = y;
-	p = ((Uint8*) image->m_data) + (y * image->m_pitch);
+	p = ((uint8_t*) image->m_data) + (y * image->m_pitch);
     }
     p[x * 4	+ 0] = iclamp(frnd(r), 0, 255);
     p[x * 4	+ 1] = iclamp(frnd(g), 0, 255);
@@ -292,9 +292,7 @@ float	Lanczos3_filter(float t)
 
 float	Mitchell_filter(float t)
 {
-    float tt;
-
-    tt = t * t;
+    float tt = t * t;
     if (t < 0.0f) t = -t;
     if (t < 1.0f) {
 	t = (((12.0f - 9.0f * B - 6.0f * C) * (t * tt))
@@ -328,7 +326,7 @@ struct CONTRIB {
 };
 
 
-};	// end anonymous namespace
+}	// end anonymous namespace
 
 
 namespace image {
@@ -389,14 +387,12 @@ void	resample(image::rgb* out, int out_x0, int out_y0, int out_x1, int out_y1,
     support = filter_table[default_type].support;
 
 
-    image::rgb*	tmp;		/* intermediate image */
-    float	xscale, yscale;		/* zoom scale factors */
     int i, k;			/* loop variables */
     unsigned int j;			/* loop variables */
     int n;				/* pixel number */
     float center; int left, right;	/* filter calculation variables */
     float width, fscale, weight;	/* filter calculation variables */
-    Uint8*	raster;			/* a row or column of pixels */
+    uint8_t*	raster;			/* a row or column of pixels */
 
     std::vector< std::vector<CONTRIB> >	contrib;
 
@@ -414,9 +410,9 @@ void	resample(image::rgb* out, int out_x0, int out_y0, int out_x1, int out_y1,
     int	in_window_h = int(ceilf(in_y1) - floorf(in_y0) + 1);
 
     /* create intermediate image to hold horizontal zoom */
-    tmp = image::create_rgb(out_width, in_window_h);
-    xscale = (float) (out_width - 1) / in_width;
-    yscale = (float) (out_height - 1) / in_height;
+    std::auto_ptr<image::rgb> tmp ( image::create_rgb(out_width, in_window_h) );
+    float xscale = (float) (out_width - 1) / in_width;
+    float yscale = (float) (out_height - 1) / in_height;
 
     // xxxx protect against division by 0
     if (yscale == 0) { yscale = 1.0f; }
@@ -456,7 +452,7 @@ void	resample(image::rgb* out, int out_x0, int out_y0, int out_x1, int out_y1,
     }
 
     /* apply filter to zoom horizontally from src to tmp */
-    raster = (Uint8*) my_calloc(in_window_w, 3);
+    raster = (uint8_t*) my_calloc(in_window_w, 3);
     for (k = 0; k < tmp->m_height; ++k) {
 	get_row(raster, in, int(floorf(in_x0)), in_window_w, k);
 	for (i = 0; i < tmp->m_width; ++i) {
@@ -469,7 +465,7 @@ void	resample(image::rgb* out, int out_x0, int out_y0, int out_x1, int out_y1,
 		green	+= raster[pixel * 3 + 1] * contrib[i][j].weight;
 		blue	+= raster[pixel * 3 + 2] * contrib[i][j].weight;
 	    }
-	    put_pixel(tmp, i, k, red, green, blue);
+	    put_pixel(tmp.get(), i, k, red, green, blue);
 	}
     }
     my_cfree(raster);
@@ -508,9 +504,9 @@ void	resample(image::rgb* out, int out_x0, int out_y0, int out_x1, int out_y1,
     }
 
     /* apply filter to zoom vertically from tmp to dst */
-    raster = (Uint8*) my_calloc(tmp->m_height, 3);
+    raster = (uint8_t*) my_calloc(tmp->m_height, 3);
     for (k = 0; k < tmp->m_width; ++k) {
-	get_column(raster, tmp, k);
+	get_column(raster, tmp.get(), k);
 	for (i = 0; i < out_height; ++i) {
 	    float	red = 0.0f;
 	    float	green = 0.0f;
@@ -528,7 +524,6 @@ void	resample(image::rgb* out, int out_x0, int out_y0, int out_x1, int out_y1,
 
     contrib.resize(0);
 
-    delete tmp;
 }
 
 
@@ -556,14 +551,13 @@ void	resample(image::rgba* out, int out_x0, int out_y0, int out_x1, int out_y1,
     support = filter_table[default_type].support;
 
 
-    image::rgba*	tmp;		/* intermediate image */
     float	xscale, yscale;		/* zoom scale factors */
     int i, k;			/* loop variables */
     unsigned int j;			/* loop variables */
     int n;				/* pixel number */
     float center; int left, right;	/* filter calculation variables */
     float width, fscale, weight;	/* filter calculation variables */
-    Uint8*	raster;			/* a row or column of pixels */
+    uint8_t*	raster;			/* a row or column of pixels */
 
     std::vector< std::vector<CONTRIB> >	contrib;
 
@@ -581,7 +575,7 @@ void	resample(image::rgba* out, int out_x0, int out_y0, int out_x1, int out_y1,
     int	in_window_h = int(ceilf(in_y1) - floorf(in_y0) + 1);
 
     /* create intermediate image to hold horizontal zoom */
-    tmp = image::create_rgba(out_width, in_window_h);
+    std::auto_ptr<image::rgba>	tmp( image::create_rgba(out_width, in_window_h) );
     xscale = (float) (out_width - 1) / in_width;
     yscale = (float) (out_height - 1) / in_height;
 
@@ -623,7 +617,7 @@ void	resample(image::rgba* out, int out_x0, int out_y0, int out_x1, int out_y1,
     }
 
     /* apply filter to zoom horizontally from src to tmp */
-    raster = (Uint8*) my_calloc(in_window_w, 4);
+    raster = (uint8_t*) my_calloc(in_window_w, 4);
     for (k = 0; k < tmp->m_height; ++k) {
 	get_row(raster, in, int(floorf(in_x0)), in_window_w, k);
 	for (i = 0; i < tmp->m_width; ++i) {
@@ -638,7 +632,7 @@ void	resample(image::rgba* out, int out_x0, int out_y0, int out_x1, int out_y1,
 		blue	+= raster[pixel * 4 + 2] * contrib[i][j].weight;
 		alpha	+= raster[pixel * 4 + 3] * contrib[i][j].weight;
 	    }
-	    put_pixel(tmp, i, k, red, green, blue, alpha);
+	    put_pixel(tmp.get(), i, k, red, green, blue, alpha);
 	}
     }
     my_cfree(raster);
@@ -677,9 +671,9 @@ void	resample(image::rgba* out, int out_x0, int out_y0, int out_x1, int out_y1,
     }
 
     /* apply filter to zoom vertically from tmp to dst */
-    raster = (Uint8*) my_calloc(tmp->m_height, 4);
+    raster = (uint8_t*) my_calloc(tmp->m_height, 4);
     for (k = 0; k < tmp->m_width; ++k) {
-	get_column(raster, tmp, k);
+	get_column(raster, tmp.get(), k);
 	for (i = 0; i < out_height; ++i) {
 	    float	red = 0.0f;
 	    float	green = 0.0f;
@@ -699,124 +693,8 @@ void	resample(image::rgba* out, int out_x0, int out_y0, int out_x1, int out_y1,
 
     contrib.resize(0);
 
-    delete tmp;
 }
 
-
-
-// tulrich: some interesting scaling code from Vitaly.  Looks like a
-// fast bilinear scale using fixed point.  I haven't validated this
-// myself.  Note: I would see about losing the sax & say arrays, and
-// fold that stuff directly into the pixel loops, to get rid of the
-// mallocs.
-
-void	zoom(image::rgba* src, image::rgba* dst)
-{
-    GNASH_REPORT_FUNCTION;
-    typedef struct
-    {
-	Uint8 r;
-	Uint8 g;
-	Uint8 b;
-	Uint8 a;
-    }
-    rgba;
-
-    int x, y, sx, sy, *sax, *say, *csax, *csay, csx, csy, ex, ey, t1, t2;
-    rgba *c00, *c01, *c10, *c11, *sp, *csp, *dp;
-    int sgap, dgap;
-
-    /* For interpolation: assume source dimension is one pixel */
-    /* smaller to avoid overflow on right and bottom edge.     */
-    sx = (int) (65536.0 * (float) (src->m_width - 1) / (float) dst->m_width);
-    sy = (int) (65536.0 * (float) (src->m_height - 1) / (float) dst->m_height);
-
-    /* Allocate memory for row increments */
-    sax = (int*) malloc ((dst->m_width + 1) * sizeof (Uint32));
-    say = (int*) malloc ((dst->m_height + 1) * sizeof (Uint32));
-
-    /* Precalculate row increments */
-    csx = 0;
-    csax = sax;
-    for (x = 0; x <= dst->m_width; x++)
-	{
-	    *csax = csx;
-	    csax++;
-	    csx &= 0xffff;
-	    csx += sx;
-	}
-    csy = 0;
-    csay = say;
-    for (y = 0; y <= dst->m_height; y++)
-	{
-	    *csay = csy;
-	    csay++;
-	    csy &= 0xffff;
-	    csy += sy;
-	}
-
-    /* Pointer setup */
-    sp = csp = (rgba *) src->m_data;
-    dp = (rgba *) dst->m_data;
-    sgap = src->m_pitch - src->m_width * 4;
-    dgap = dst->m_pitch - dst->m_width * 4;
-
-    /* Interpolating Zoom */
-    /* Scan destination */
-    csay = say;
-    for (y = 0; y < dst->m_height; y++)
-	{
-	    /* Setup color source pointers */
-	    c00 = csp;
-	    c01 = csp;
-	    c01++;
-	    c10 = (rgba *) ((Uint8 *) csp + src->m_pitch);
-	    c11 = c10;
-	    c11++;
-	    csax = sax;
-	    for (x = 0; x < dst->m_width; x++)
-		{
-		    /* ABGR ordering */
-		    /* Interpolate colors */
-		    ex = (*csax & 0xffff);
-		    ey = (*csay & 0xffff);
-		    t1 = ((((c01->r - c00->r) * ex) >> 16) + c00->r) & 0xff;
-		    t2 = ((((c11->r - c10->r) * ex) >> 16) + c10->r) & 0xff;
-		    dp->r = (((t2 - t1) * ey) >> 16) + t1;
-		    t1 = ((((c01->g - c00->g) * ex) >> 16) + c00->g) & 0xff;
-		    t2 = ((((c11->g - c10->g) * ex) >> 16) + c10->g) & 0xff;
-		    dp->g = (((t2 - t1) * ey) >> 16) + t1;
-		    t1 = ((((c01->b - c00->b) * ex) >> 16) + c00->b) & 0xff;
-		    t2 = ((((c11->b - c10->b) * ex) >> 16) + c10->b) & 0xff;
-		    dp->b = (((t2 - t1) * ey) >> 16) + t1;
-		    t1 = ((((c01->a - c00->a) * ex) >> 16) + c00->a) & 0xff;
-		    t2 = ((((c11->a - c10->a) * ex) >> 16) + c10->a) & 0xff;
-		    dp->a = (((t2 - t1) * ey) >> 16) + t1;
-
-		    /* Advance source pointers */
-		    csax++;
-		    int sstep = (*csax >> 16);
-		    c00 += sstep;
-		    c01 += sstep;
-		    c10 += sstep;
-		    c11 += sstep;
-		    /* Advance destination pointer */
-		    dp++;
-		}
-	    /* Advance source pointer */
-	    csay++;
-	    csp = (rgba *) ((Uint8 *) csp + (*csay >> 16) * src->m_pitch);
-	    /* Advance destination pointers */
-	    dp = (rgba *) ((Uint8 *) dp + dgap);
-	}
-
-    /* Remove temp arrays */
-    free (sax);
-    free (say);
-}
-
-
-
 } // end namespace image
 
 
