@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -130,6 +130,10 @@ public:
 	/// @param varname 
 	///	Variable name. Can contain path elements.
 	///	TODO: should be case-insensitive up to SWF6.
+	///	NOTE: no case conversion is performed currently,
+	///	      so make sure you do it yourself. Note that
+	///	      ActionExec performs the conversion
+	///	      before calling this method.
 	///
 	as_value get_variable(const std::string& varname) const;
 
@@ -140,6 +144,10 @@ public:
 	/// @param varname 
 	///	Variable name. Can not contain path elements.
 	///	TODO: should be case-insensitive up to SWF6.
+	///	NOTE: no case conversion is performed currently,
+	///	      so make sure you do it yourself. Note that
+	///	      ActionExec performs the conversion
+	///	      before calling this method.
 	///
 	/// @param scopeStack
 	///	The Scope stack to use for lookups.
@@ -152,6 +160,10 @@ public:
 	/// @param varname 
 	///	Variable name. Can contain path elements.
 	///	TODO: should be case-insensitive up to SWF6.
+	///	NOTE: no case conversion is performed currently,
+	///	      so make sure you do it yourself. Note that
+	///	      ActionExec performs the conversion
+	///	      before calling this method.
 	///
 	/// @param scopeStack
 	///	The Scope stack to use for lookups.
@@ -170,6 +182,10 @@ public:
 	/// @param path 
 	///	Variable path.
 	///	TODO: should be case-insensitive up to SWF6.
+	///	NOTE: no case conversion is performed currently,
+	///	      so make sure you do it yourself. Note that
+	///	      ActionExec performs the conversion
+	///	      before calling this method.
 	///
 	/// @param val
 	///	The value to assign to the variable, if found.
@@ -313,6 +329,16 @@ public:
 	    }
 	}
 
+#ifdef GNASH_USE_GC
+	/// Mark all reachable resources.
+	//
+	/// Reachable resources from an as_environment
+	/// would be global registers, stack (expected to be empty
+	/// actually), stack frames and targets (original and current).
+	///
+	void markReachableResources() const;
+#endif
+
 	/// Find the sprite/movie referenced by the given path.
 	//
 	/// Supports both /slash/syntax and dot.syntax
@@ -430,6 +456,14 @@ public:
 		Registers registers;
 
 		as_function* func;
+
+#ifdef GNASH_USE_GC
+		/// Mark all reachable resources
+		//
+		/// Reachable resources would be registers and
+		/// locals (expected to be empty?) and function.
+		void markReachableResources() const;
+#endif // GNASH_USE_GC
 	};
 
 	/// Push a frame on the calls stack.
@@ -496,7 +530,8 @@ private:
 	//
 	/// @param varname 
 	///	Variable name. Can not contain path elements.
-	///	TODO: should be case-insensitive up to SWF6.
+	///	NOTE: no case conversion is performed currently,
+	///	      so make sure you do it yourself. 
 	///
 	as_value get_variable_raw(const std::string& varname) const;
 
