@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -125,8 +125,39 @@ public:
 	/// Return the built-in Function constructor
 	static boost::intrusive_ptr<builtin_function> getFunctionConstructor();
 
+#ifdef GNASH_USE_GC
+	/// Mark reachable resources. Override from GcResource
+	//
+	/// Reachable resources from this object is its prototype
+	/// and the default as_object reachables (properties and parent).
+	///
+	/// The default implementation only marks that. If you
+	/// override this function from a derived class remember
+	/// to call markAsFunctionReachableResources() as a final step.
+	///
+	virtual void markReachableResources() const
+	{
+		markAsFunctionReachable();
+	}
+#endif // GNASH_USE_GC
+
 protected:
 
+#ifdef GNASH_USE_GC
+	/// Mark prototype (properties) as being reachable and invoke
+	/// the as_object class marker.
+	//
+	/// Call this function from an override of markReachableResources
+	/// in a derived class
+	///
+	void markAsFunctionReachable() const
+	{
+		_properties->setReachable();
+
+		markAsObjectReachable();
+	}
+#endif // GNASH_USE_GC
+
 	/// Construct a function with given interface
 	//
 	/// If the given interface is NULL a default one
