@@ -4,7 +4,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: xml.cpp,v 1.42 2007/05/14 16:24:44 strk Exp $ */
+/* $Id: xml.cpp,v 1.44 2007/07/01 10:54:32 bjacques Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -583,7 +583,7 @@ xml_new(const fn_call& fn)
             xml_obj = boost::dynamic_pointer_cast<XML>(obj);
             if ( xml_obj )
             {
-                log_msg(_("\tCloned the XML object at %p"), xml_obj.get());
+                log_msg(_("\tCloned the XML object at %p"), (void *)xml_obj.get());
                 return as_value(xml_obj->cloneNode(true).get());
             }
         }
