@@ -4,7 +4,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -268,7 +268,11 @@ button_character_instance::button_character_instance(
 	{
 		if (m_def->m_button_actions[i].m_conditions & 0xFE00)	// check up on CondKeyPress: UB[7]
 		{
+#ifdef NEW_KEY_LISTENER_LIST_DESIGN
+			_vm.getRoot().add_key_listener(KeyListener(this, KeyListener::ON_CLIP_DEF));
+#else
 			_vm.getRoot().add_key_listener(this);
+#endif
 			break;
 		}
 	}
@@ -349,10 +353,7 @@ button_character_instance::on_event(const event_id& id)
 
 	bool called = false;
 
-	sprite_instance* parent = get_parent()->to_movie();
-	assert ( parent );
-
-	// Add appropriate actions to the movie's execute list...
+	// Add appropriate actions to the global action list ...
 	// TODO: should we execute immediately instead ?
 	for (size_t i = 0, ie=m_def->m_button_actions.size(); i<ie; ++i)
 	{
@@ -366,7 +367,6 @@ button_character_instance::on_event(const event_id& id)
 			for (size_t j=0, je=ba.m_actions.size(); j<je; ++j)
 			{
 				VM::get().getRoot().pushAction(*(ba.m_actions[j]), boost::intrusive_ptr<character>(this));
-				//parent->add_action_buffer(ba.m_actions[j]);
 			}
 			called = true;
 		}
@@ -871,6 +871,7 @@ button_character_instance::construct()
 
 		// we don't need an id here, do we ?
 		boost::intrusive_ptr<character> ch = bdef.m_character_def->create_character_instance(this, 0);
+		ch->construct(); // give this character life
 		m_record_character[r] = ch;
 		ch->set_matrix(mat);
 		ch->set_cxform(cx);
@@ -889,6 +890,26 @@ button_character_instance::construct()
 	// We'll not call it here..
 }
 
+#ifdef GNASH_USE_GC
+void
+button_character_instance::markReachableResources() const
+{
+	assert(isReachable());
+
+	m_def->setReachable();
+
+	// Markstate characters as reachable
+	for (CharsVect::const_iterator i=m_record_character.begin(), e=m_record_character.end();
+			i!=e; ++i)
+	{
+		(*i)->setReachable();
+	}
+
+	// character class members
+	markCharacterReachable();
+}
+#endif // GNASH_USE_GC
+
 } // end of namespace gnash
 
 
