@@ -3,7 +3,7 @@
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -57,6 +57,7 @@
 
 #include <string>
 #include <memory> // for auto_ptr
+#include <map>
 
 // Forward declarations
 namespace gnash {
@@ -325,15 +326,23 @@ public:
 	///
 	/// @see add_font
 	///
-	virtual font* get_font(int /*id*/)
+	virtual font* get_font(int /*id*/) const
 	{
 		return NULL;
 	}
 
-	/// Add an execute_tag to the frame currently being loaded
+	/// Add an execute_tag to this movie_definition's playlist
 	//
 	/// The default implementation is a no-op.
 	///
+	/// @param tag
+	/// 	The tag to add in the list of executable tags for
+	/// 	the frame currently being loaded. Ownership is transferred
+	/// 	to the movie_def_impl.
+	///
+	/// TODO: take an auto_ptr.
+	/// NOTE: the default implementation just let the execute_tag leak.
+	///
 	virtual void	add_execute_tag(execute_tag* /*c*/)
 	{
 	}
@@ -569,7 +578,9 @@ public:
 	///	Depth of an instance placed by PlaceObject* tag.
 	///	Assumed to be in the static zone (an assertion would fail otherwise).
 	///
-	virtual void addTimelineDepth(int /*depth*/) {}
+	/// @param ratio
+	/// Ratio of an instance defined by PlaceObject2 tag, ranges from 0 to 65535
+	virtual void addTimelineDepth(int /*depth*/, int /*ratio*/) {}
 
 	/// Remove a depth from the timeline depth set in current frame 
 	//
@@ -601,10 +612,9 @@ public:
 	/// @param depths
 	///	A vector to copy all frame depths to.
 	///
-	virtual void getTimelineDepths(size_t /*frameno*/,
-			std::vector<int>& /*depths*/)
+	virtual void getTimelineDepths(size_t /*frameno*/, std::map<int, int>& /*depths*/) 
 	{}
-
+    
 };
 
 } // namespace gnash
