@@ -4,7 +4,7 @@
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: shape_character_def.cpp,v 1.27 2007/05/24 13:42:50 strk Exp $ */
+/* $Id: shape_character_def.cpp,v 1.32 2007/08/07 20:53:10 strk Exp $ */
 
 // Based on the public domain shape.cpp of Thatcher Ulrich <tu@tulrich.com> 2003
 
@@ -30,7 +30,6 @@
 #include "render.h"
 #include "stream.h"
 #include "sprite_instance.h"
-#include "GnashException.h"
 
 #include "tu_file.h"
 
@@ -72,11 +71,13 @@ read_fill_styles(std::vector<fill_style>& styles, stream* in,
 {
 
 	// Get the count.
+	in->ensureBytes(1);
 	uint16_t fill_style_count = in->read_u8();
 	if (tag_type > 2)
 	{
 		if (fill_style_count == 0xFF)
 		{
+			in->ensureBytes(2);
 			fill_style_count = in->read_u16();
 		}
 	}
@@ -92,16 +93,7 @@ read_fill_styles(std::vector<fill_style>& styles, stream* in,
 		// TODO: add a fill_style constructor directly
 		//       reading from stream
 		fill_style fs;
-		try
-		{
-			fs.read(in, tag_type, m);
-		}
-		catch (ParserException& e)
-		{
-			IF_VERBOSE_MALFORMED_SWF(
-				log_swferror("%s", e.what());
-			);
-		}
+		fs.read(in, tag_type, m);
 		// Push a style anyway, so any path referring to
 		// it still finds it..
 		styles.push_back(fs);
@@ -115,6 +107,7 @@ read_line_styles(std::vector<line_style>& styles, stream* in, int tag_type)
     // Read line styles and push them onto the back of the given array.
 {
     // Get the count.
+    in->ensureBytes(1);
     int	line_style_count = in->read_u8();
 
 		IF_VERBOSE_PARSE
@@ -126,6 +119,7 @@ read_line_styles(std::vector<line_style>& styles, stream* in, int tag_type)
     // if (tag_type > 2)
     // {
     if (line_style_count == 0xFF) {
+        in->ensureBytes(2);
 	line_style_count = in->read_u16();
 		IF_VERBOSE_PARSE
 		(
@@ -145,27 +139,31 @@ read_line_styles(std::vector<line_style>& styles, stream* in, int tag_type)
 
 shape_character_def::shape_character_def()
 	:
+	character_def(),
 	m_fill_styles(),
 	m_line_styles(),
 	m_paths(),
-	m_bound(),
-	m_cached_meshes()
+	m_bound()
+	//,m_cached_meshes()
 {
 }
 
-
-void
-shape_character_def::clear_meshes()
+shape_character_def::shape_character_def(const shape_character_def& o)
+	:
+	character_def(o),
+	tesselate::tesselating_shape(o),
+	m_fill_styles(o.m_fill_styles),
+	m_line_styles(o.m_line_styles),
+	m_paths(o.m_paths),
+	m_bound(o.m_bound)
+	//,m_cached_meshes()
 {
-    // Free our mesh_sets.
-    for (unsigned int i = 0; i < m_cached_meshes.size(); i++) {
-	delete m_cached_meshes[i];
-    }
 }
 
+
 shape_character_def::~shape_character_def()
 {
-	clear_meshes();
+	//clear_meshes();
 }
 
 
@@ -296,7 +294,7 @@ shape_character_def::read(stream* in, int tag_type, bool with_style,
 			if ( style > m_fill_styles.size() ) // 1-based index 
 			{
 				IF_VERBOSE_MALFORMED_SWF(
-				log_swferror(_("Invalid fill style %d in fillStyle0Change record - %u defined. Set to 0."), style, m_fill_styles.size());
+				log_swferror(_("Invalid fill style %d in fillStyle0Change record - " SIZET_FMT " defined. Set to 0."), style, m_fill_styles.size());
 				);
 				style = 0;
 			}
@@ -339,7 +337,7 @@ shape_character_def::read(stream* in, int tag_type, bool with_style,
 			if ( style > m_fill_styles.size() ) // 1-based index 
 			{
 				IF_VERBOSE_MALFORMED_SWF(
-				log_swferror(_("Invalid fill style %d in fillStyle1Change record - %u defined. Set to 0."), style, m_fill_styles.size());
+				log_swferror(_("Invalid fill style %d in fillStyle1Change record - " SIZET_FMT " defined. Set to 0."), style, m_fill_styles.size());
 				);
 				style = 0;
 			}
@@ -379,7 +377,7 @@ shape_character_def::read(stream* in, int tag_type, bool with_style,
 			if ( style > m_line_styles.size() ) // 1-based index 
 			{
 				IF_VERBOSE_MALFORMED_SWF(
-				log_swferror(_("Invalid fill style %d in lineStyleChange record - %u defined. Set to 0."), style, m_line_styles.size());
+				log_swferror(_("Invalid fill style %d in lineStyleChange record - " SIZET_FMT " defined. Set to 0."), style, m_line_styles.size());
 				);
 				style = 0;
 			}
@@ -796,6 +794,17 @@ void	shape_character_def::compute_bound(rect* r) const
 }
 
 
+#if 0 // deprecated
+
+void
+shape_character_def::clear_meshes()
+{
+    // Free our mesh_sets.
+    for (unsigned int i = 0; i < m_cached_meshes.size(); i++) {
+	delete m_cached_meshes[i];
+    }
+}
+
 void	shape_character_def::output_cached_data(tu_file* out, const cache_options& /* options */)
     // Dump our precomputed mesh data to the given stream.
 {
@@ -822,6 +831,21 @@ void	shape_character_def::input_cached_data(tu_file* in)
     }
 }
 
+#endif // deprecated cached data
+
+#ifdef GNASH_USE_GC
+void
+shape_character_def::markReachableResources() const
+{
+	assert(isReachable());
+	for (FillStyleVect::const_iterator i=m_fill_styles.begin(), e=m_fill_styles.end();
+			i != e; ++i)
+	{
+		i->markReachableResources();
+	}
+}
+#endif // GNASH_USE_GC
+
 }	// end namespace gnash
 
 // Local Variables:
